@@ -19,25 +19,40 @@ from contextlib import contextmanager
 
 from ufo2ft import CFFOptimization
 from ufo2ft.featureWriters import loadFeatureWriterFromString
+from ufo2ft.filters import loadFilterFromString
 
 from fontmake import __version__
 from fontmake.errors import FontmakeError
 from fontmake.font_project import INTERPOLATABLE_OUTPUTS, FontProject
 
 
-def _loadFeatureWriters(parser, specs):
-    feature_writers = []
+def _loadPlugins(parser, specs, from_string_func, parser_error_message):
+    plugins = []
     for s in specs:
         if s == "None":
-            # magic value that means "don't generate any features!"
+            # magic value that means "don't write any features or don't apply
+            # any filters!"
             return []
         try:
-            feature_writers.append(loadFeatureWriterFromString(s))
+            plugins.append(from_string_func(s))
         except Exception as e:
-            parser.error(
-                "Failed to load --feature-writer:\n  {}: {}".format(type(e).__name__, e)
-            )
-    return feature_writers
+            parser.error(parser_error_message.format(type(e).__name__, e))
+    return plugins
+
+
+def _loadFeatureWriters(parser, specs):
+    return _loadPlugins(
+        parser,
+        specs,
+        loadFeatureWriterFromString,
+        "Failed to load --feature-writer:\n  {}: {}",
+    )
+
+
+def _loadFilters(parser, specs):
+    return _loadPlugins(
+        parser, specs, loadFilterFromString, "Failed to load --filter:\n  {}: {}"
+    )
 
 
 def exclude_args(parser, args, excluded_args, target, positive=True):
@@ -225,6 +240,12 @@ def main(args=None):
         "interpolating. Use if you share feature files of masters in "
         "external files, as instances can end up elsewhere.",
     )
+    outputGroup.add_argument(
+        "--no-generate-GDEF",
+        dest="generate_GDEF",
+        action="store_false",
+        help="Do not auto-generate a GDEF table, but keep an existing one intact.",
+    )
 
     contourGroup = parser.add_argument_group(title="Handling of contours")
     contourGroup.add_argument(
@@ -304,6 +325,18 @@ def main(args=None):
         help="Do not perform IUP optimization on variable font's 'gvar' table. "
         "(only works with 'variable' TrueType-flavored output)",
     )
+    contourGroup.add_argument(
+        "--filter",
+        metavar="CLASS",
+        action="append",
+        dest="filter_specs",
+        help="string specifying a filter class to load, either "
+        "built-in or from an external module, optionally initialized with "
+        "the given keyword arguments. The class and module names are "
+        "separated by '::'. The option can be repeated multiple times "
+        "for each filter class. The option overrides the filters specified "
+        "in the UFO lib.",
+    )
 
     layoutGroup = parser.add_argument_group(title="Handling of OpenType Layout")
     layoutGroup.add_argument(
@@ -398,6 +431,10 @@ def main(args=None):
     if specs is not None:
         args["feature_writers"] = _loadFeatureWriters(parser, specs)
 
+    specs = args.pop("filter_specs")
+    if specs is not None:
+        args["filters"] = _loadFilters(parser, specs)
+
     glyphs_path = args.pop("glyphs_path")
     ufo_paths = args.pop("ufo_paths")
     designspace_path = args.pop("mm_designspace")
@@ -484,6 +521,10 @@ def main(args=None):
             logging.exception(e)
             sys.exit(1)
         sys.exit(f"fontmake: Error: {str(e)}")
+    finally:
+        debug_feature_file = args.get("debug_feature_file")
+        if debug_feature_file is not None:
+            debug_feature_file.close()
 
 
 if __name__ == "__main__":
