@@ -3,6 +3,7 @@ import logging
 import fontTools.designspaceLib as designspaceLib
 import pytest
 import ufoLib2
+from fontTools.pens.recordingPen import RecordingPen
 from ufoLib2.objects.anchor import Anchor
 
 import fontmake.instantiator
@@ -60,6 +61,63 @@ def test_interpolation_weight_width_class(data_dir):
     assert font.info.openTypeOS2WidthClass == 9
 
 
+def test_default_groups_only(data_dir, caplog):
+    """Test that only the default source's groups end up in instances."""
+
+    d = designspaceLib.DesignSpaceDocument()
+    d.addAxisDescriptor(
+        name="Weight", tag="wght", minimum=300, default=300, maximum=900
+    )
+    d.addSourceDescriptor(location={"Weight": 300}, font=ufoLib2.Font())
+    d.addSourceDescriptor(location={"Weight": 900}, font=ufoLib2.Font())
+    d.addInstanceDescriptor(styleName="2", location={"Weight": 400})
+    d.findDefault()
+
+    d.sources[0].font.groups["public.kern1.GRK_alpha_alt_LC_1ST"] = [
+        "alpha.alt",
+        "alphatonos.alt",
+    ]
+    d.sources[1].font.groups["public.kern1.GRK_alpha_LC_1ST"] = [
+        "alpha.alt",
+        "alphatonos.alt",
+    ]
+
+    generator = fontmake.instantiator.Instantiator.from_designspace(d)
+    assert "contains different groups than the default source" in caplog.text
+
+    instance = generator.generate_instance(d.instances[0])
+    assert instance.groups == {
+        "public.kern1.GRK_alpha_alt_LC_1ST": ["alpha.alt", "alphatonos.alt"]
+    }
+
+
+def test_default_groups_only2(data_dir, caplog):
+    """Test that the group difference warning is not triggered if non-default
+    source groups are empty."""
+
+    d = designspaceLib.DesignSpaceDocument()
+    d.addAxisDescriptor(
+        name="Weight", tag="wght", minimum=300, default=300, maximum=900
+    )
+    d.addSourceDescriptor(location={"Weight": 300}, font=ufoLib2.Font())
+    d.addSourceDescriptor(location={"Weight": 900}, font=ufoLib2.Font())
+    d.addInstanceDescriptor(styleName="2", location={"Weight": 400})
+    d.findDefault()
+
+    d.sources[0].font.groups["public.kern1.GRK_alpha_alt_LC_1ST"] = [
+        "alpha.alt",
+        "alphatonos.alt",
+    ]
+
+    generator = fontmake.instantiator.Instantiator.from_designspace(d)
+    assert "contains different groups than the default source" not in caplog.text
+
+    instance = generator.generate_instance(d.instances[0])
+    assert instance.groups == {
+        "public.kern1.GRK_alpha_alt_LC_1ST": ["alpha.alt", "alphatonos.alt"]
+    }
+
+
 def test_interpolation_no_rounding(data_dir):
     designspace = designspaceLib.DesignSpaceDocument.fromfile(
         data_dir / "MutatorSans" / "MutatorSans.designspace"
@@ -199,7 +257,7 @@ def test_swap_glyph_names_spec(data_dir):
 
     `adieresis` should look the same as before the rule application.
 
-    [1]: https://github.com/fonttools/fonttools/tree/master/Doc/source/designspaceLib#ufo-instances
+    [1]: fonttools/Doc/source/designspaceLib#ufo-instances
     """
     ufo = ufoLib2.Font.open(data_dir / "SwapGlyphNames" / "B.ufo")
     fontmake.instantiator.swap_glyph_names(ufo, "a", "a.alt")
@@ -214,6 +272,40 @@ def test_swap_glyph_names_spec(data_dir):
     ]
 
 
+def test_rules_are_applied_deterministically(data_dir):
+    """Test that a combination of designspace rules that end up mapping
+    serveral input glyphs to the same destination glyph result in a correct and
+    deterministic series of glyph swaps.
+
+    The example is a font with 2 Q designs that depend on a style axis
+        style < 0.5: Q        style >= 0.5: Q.ss01
+    and each Q also has an alternative shape in bolder weights (like Skia)
+        weight < 780: Q       weight >= 780: Q.alt
+        weight < 730: Q.ss01  weight >= 730: Q.ss01.alt
+
+    Then we generate an instance at style = 1, weight = 900. From the rules,
+    the default CMAP entry for Q should have the outlines of Q.ss01.alt from
+    the black UFO.
+    """
+    doc = designspaceLib.DesignSpaceDocument.fromfile(
+        data_dir / "DesignspaceRuleOrder" / "MyFont.designspace"
+    )
+    instanciator = fontmake.instantiator.Instantiator.from_designspace(doc)
+    instance = instanciator.generate_instance(doc.instances[0])
+    pen = RecordingPen()
+    instance["Q"].draw(pen)
+    instance_recording = pen.value
+
+    black_ufo = ufoLib2.Font.open(
+        data_dir / "DesignspaceRuleOrder" / "MyFont_Black.ufo"
+    )
+    pen = RecordingPen()
+    black_ufo["Q.ss01.alt"].draw(pen)
+    black_ufo_recording = pen.value
+
+    assert instance_recording == black_ufo_recording
+
+
 def test_raise_no_default_master(data_dir):
     designspace = designspaceLib.DesignSpaceDocument.fromfile(
         data_dir / "MutatorSans" / "MutatorSans_no_default.designspace"
@@ -291,6 +383,25 @@ def test_interpolation(data_dir):
     assert instance_font["l"].width == 220
 
 
+def test_interpolation_only_default(data_dir, caplog):
+    designspace = designspaceLib.DesignSpaceDocument.fromfile(
+        data_dir / "MutatorSans" / "MutatorSans.designspace"
+    )
+    designspace.loadSourceFonts(ufoLib2.Font.open)
+    for name in designspace.default.font.glyphOrder:
+        if name != "A":
+            del designspace.default.font[name]
+
+    with caplog.at_level(logging.WARNING):
+        generator = fontmake.instantiator.Instantiator.from_designspace(
+            designspace, round_geometry=True
+        )
+    assert "contains glyphs that are missing from the" in caplog.text
+
+    instance_font = generator.generate_instance(designspace.instances[0])
+    assert {g.name for g in instance_font} == {"A"}
+
+
 def test_interpolation_masters_as_instances(data_dir):
     designspace = designspaceLib.DesignSpaceDocument.fromfile(
         data_dir
@@ -503,8 +614,8 @@ def test_data_independence(data_dir):
 
 def test_skipped_fontinfo_attributes():
     """Test that we consider all available font info attributes for copying."""
-    import fontTools.ufoLib
     import fontMath.mathInfo
+    import fontTools.ufoLib
 
     SKIPPED_ATTRS = {
         "guidelines",
